----=============================================================================================================

local addonName = ...
local VERSION = GetAddOnMetadata(addonName, "Version");

----=============================================================================================================

local HQT = CreateFrame("GameTooltip", "HiddenQuestTracker", UIParent, "GameTooltipTemplate")
HQT:SetScript("OnEvent", function(self, event, ...) self:OnEvent(self, event, ...) end)
HQT:RegisterEvent("ADDON_LOADED") -- Needed for the position to get restored!
--HQT:RegisterEvent("CRITERIA_UPDATE")
HQT:RegisterEvent("QUEST_LOG_UPDATE")

UIParent:HookScript("OnShow", function() HQT:Update() end)

--=============================================================================================================

function HQT:OnEvent(self, event, ...)

	--DEFAULT_CHAT_FRAME:AddMessage(format("--- %s --- %s |cffFFFFFF- %s", self:GetName(), GetTime(), event), 1, 0, 0)

	if event == "ADDON_LOADED" then
		if select(1, ...) == addonName then
			DEFAULT_CHAT_FRAME:AddMessage(format("[%s] v%s loaded", addonName, VERSION), 1, .48, 0)
			self:Init()
			--self:Update()
		end
		return
	end

	----------------------------------------------------------------
    	
	if event == "QUEST_LOG_UPDATE" then
		--PlaySoundFile("Sound/doodad/fx_chimesingle_01.ogg")
		C_Timer.After(0.5, function() self:Update() end)
		return
	end

end

--=============================================================================================================

function HQT:Init()

	if not self:IsUserPlaced() then
		self:SetPoint("CENTER")
	end

	self:SetMovable(true) -- This must be run on "ADDON_LOADED" for the position to get restored!
	self:RegisterForDrag("LeftButton")
	self:SetScript("OnMouseDown", 
		function(self, button)
			if button == "LeftButton" then
				self:StartMoving()
			elseif button == "RightButton" then
				self:EnterQuestIDs()
			elseif button == "MiddleButton" then
				self.skin_nr = self.skin_nr + 1
				HiddenQuestTracker_Settings.skin_nr = self.skin_nr
				self:ChangeSkin()
			end
			self:Update()
		end)

	self:SetScript("OnMouseUp", self.StopMovingOrSizing)

	self:SetScript("OnMouseWheel", 
		function(self, value) 
			if ( IsAltKeyDown() ) then
				self:SetAlpha(self:GetAlpha() + value / 10)
			--elseif ( IsControlKeyDown() and IsShiftKeyDown() ) then -- Reset
			--elseif ( IsControlKeyDown() ) then
				--self:SetScale(self:GetScale() + value / 20)
			else
				self:SetScale(self:GetScale() + value / 20)
			end
		end)

	if HiddenQuestTracker_Settings == nil then
		HiddenQuestTracker_Settings = { }
	end

	self.questList = HiddenQuestTracker_Settings.questList or { }
	self.skin_nr = HiddenQuestTracker_Settings.skin_nr or 1
--	self:SetScale(1.8)
	self:ChangeSkin()

end

--=============================================================================================================

function HQT:Update()

	self:SetOwner(UIParent, "ANCHOR_PRESERVE")
--	self:SetScale(.8)
	self:ClearLines()
--	self:SetText(addonName)
	self:SetText("< H Q T >")
--	self:SetText(GetTime())

--  UIFrameFadeOut(self, 0.5, 1, 0)
--  UIFrameFadeIn(self, 0.5, 0, 1)
  UIFrameFadeOut(HiddenQuestTrackerTextLeft1, 0.5, 1, 0)
  UIFrameFadeIn(HiddenQuestTrackerTextLeft1, 0.5, 0, 1)

	if HiddenQuestTracker_Settings.questList == nil or HiddenQuestTracker_Settings.questList == {} then
		self:AddLine("Empty list!")
	else
		for i, id in pairs(HiddenQuestTracker_Settings.questList) do
			if tonumber(id) ~= nil then
				self:AddLine(((i < 10 and #HiddenQuestTracker_Settings.questList > 9) and "  " or "") .. i .. " - " .. (IsQuestFlaggedCompleted(id) and "|cff00FF00" or "|cffFF0000") .. id .. "|r", 1, 1, 1)

--				self:AddDoubleLine(((i < 10 and #HiddenQuestTracker_Settings.questList > 9) and "  " or "") .. i, (IsQuestFlaggedCompleted(id) and "|cff00FF00" or "|cffFF0000") .. id .. "|r", 1, 1, 1)
--				self:AddDoubleLine(((i < 10 and #HiddenQuestTracker_Settings.questList > 9) and "  " or "") .. i .. " - |cff777777" .. id .. "|r", IsQuestFlaggedCompleted(id) and "|cff00FF00Yes|r" or "|cffFF0000No|r", 1, 1, 1)
			else
				self:AddLine(i .. " - Invalid ID", 1, 0, 0)

--				self:AddDoubleLine(((i < 10 and #HiddenQuestTracker_Settings.questList > 9) and "  " or "") .. i .. " - |cff777777" .. id .. "|r", "|cffFF0000Invalid ID|r", 1, 0, 0)
			end
		end
	end

	self:Show()
--	self:ChangeSkin()

	--print(format("|cff00FFFFMiniCurrency CPU|r %.2f ms", GetFrameCPUUsage(self, true)))

end

--=============================================================================================================

function HQT:SaveQuestIDs(questList)
--local list = { strsplit(",", questList) }
--alerttable(list)
	if questList == "" then return end
	HiddenQuestTracker_Settings.questList = { strsplit(",", questList) }
	self:Update()
end

--=============================================================================================================

function HQT:EnterQuestIDs()
	StaticPopupDialogs["HQT_ENTERQUESTIDS"] = {
		text = "|cffFFFF00Enter a comma separated list of quest id's:|r",
		button1 = ACCEPT,
		button2 = CANCEL,
		hasEditBox = 1,
		editBoxWidth = 800,
		--maxLetters = 128,
		wide = true,
		OnAccept = function(self)
			HQT:SaveQuestIDs(self.editBox:GetText())
		end,
		OnShow = function(self)
			C_Timer.After(0.01, function() self:SetWidth(870) end)
			self.editBox:SetText(table.concat(HiddenQuestTracker_Settings.questList or {}, ","))
			self.editBox:SetCursorPosition(0)
		end,

		--OnHide = function(self) end,

		EditBoxOnEnterPressed = function(self)
			HQT:SaveQuestIDs(self:GetParent().editBox:GetText())
			self:GetParent():Hide()
		end,

		EditBoxOnEscapePressed = function(self) self:GetParent():Hide() end,
		timeout = 0,
		exclusive = 1,
		whileDead = 1,
		hideOnEscape = 1,
		preferredIndex = 3,  -- avoid some UI taint, see http://www.wowace.com/announcements/how-to-avoid-some-ui-taint/
	}

	StaticPopup_Show("HQT_ENTERQUESTIDS");
end

--=============================================================================================================

function HQT:ChangeSkin()
	if not self.skin_nr or self.skin_nr > 4 then
		self.skin_nr = 0
	end
	
	local backdrop = {
			bgFile = "Interface/Tooltips/UI-Tooltip-Background",
			edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
			tile = true,
			tileSize = 16,
			edgeSize = 16,
			insets = { left = 4, right = 4, top = 4, bottom = 4, }
			}
	
	if self.skin_nr == 0 then
		self:SetBackdrop(backdrop)
		self:SetBackdropColor(0.09, 0.09, 0.19, 0.99)

	elseif self.skin_nr == 1 then
		self:SetBackdrop(backdrop)
		self:SetBackdropColor(0, 0, 0, 1)
		self:SetBackdropBorderColor(0, 0, 0, 0.4)

	elseif self.skin_nr == 2 then
		self:SetBackdrop(backdrop)
		self:SetBackdropColor(0, 0, 0, 0.5)
		self:SetBackdropBorderColor(0, 0, 0, 0.2)
	
	elseif self.skin_nr == 3 then
		backdrop.bgFile = nil
		self:SetBackdrop(backdrop)
		self:SetBackdropBorderColor(0, 0, 0, 0.2)
	
	elseif self.skin_nr == 4 then
		backdrop.bgFile = nil
		backdrop.edgeFile = nil
		self:SetBackdrop(backdrop)

	end
end

--=============================================================================================================
